#!/usr/local/bin/perl

# Copyright (C) 2003 by Johannes Grassler. All rights reserved. This is
# free software and can be distributed under the terms of either the
# Artistic License or the GNU General Public License which can be found
# in the files Artistic and COPYING.

use strict;
use warnings;

use Getopt::Std;
use Digest::MD5 qw(md5 md5_hex);

my %opts;
getopts('a:i:o:f:G:h', \%opts);

my $usage = "usage: $0 -o <output file> -f <MD5 sum] field in GECOS> -G <affected group> -a <file with sums to ignore> [input file]\n";
if ( $opts{h} ) { die $usage; }

# Set the GID for the affected users
my $target_gid = 2001;
if ( $opts{G} ) 
	{ 
	if ( $opts{G} =~ /^\d+$/ ) 
		{ 
		if ( ! getgrgid($opts{G}) ) { die "Group $opts{G} doesn't exist.\n"; }
		$target_gid = $opts{G};
		}
	else
		{ 
		if ( ! getgrnam($opts{G}) ) { die "Group $opts{G} doesn't exist.\n"; }
		$target_gid = $opts{G}; 
		}
	}

# Set the list of users who won't be marked for deletion even if their
# sums are not found in the input file

my $input = "-";
if ( $ARGV[0]) { $input = $ARGV[0]; }

my $output = "-";
if ( $opts{o} ) { $output = $opts{o}; }

my $field = 2;
if ( $opts{f} )
	{ 
	if ( $opts{f} !~ /^\d+$/ ) { die "Argument to -f is invalid. Please specify a numeric value.\n"; }
	else { $field = $opts{f} - 1; }
	}



# Read the list of MD5 sums to be checked against
my %users;
open INPUT,$input or die "Couldn't open $input: $!\n";
while ( my $line = <INPUT> )
	{
	chomp $line;
	my ($user, $realname, $class, $checksum) = split(/\t/, $line);
	$users{$checksum}= 
			{ 
			'name' => $user,
			'realname' => $realname,
			'class' => $class,
			'checksum' => $checksum
			};
	}

my @ignoresums;
if ( $opts{a} ) 
	{ 
	my $ignoresums_file;
	$ignoresums_file = $opts{a};

	# get the checksums that are to be ignored
	open IGNORESUMS,$ignoresums_file or die "Couldn't open $ignoresums_file: $!\n" ;
	while ( my $line = <IGNORESUMS> )
		{
		chomp $line;
		push @ignoresums, $line;
		}
	close IGNORESUMS;
	}


# open the output filehandle

open OUTPUT,">$output";

# Iterate through the passwd file
PWDPARSE: while ( my ($username, undef, undef, $gid, undef, undef, $gecos) = getpwent() )
	{
	# First check if the user is in the target group and skip to the
	# next iteration if not
	if ( $gid ne $target_gid ) { next PWDPARSE; }


	# Now check the sum of the user entry against the sum from the list
	my @gecos_entries = split(/;/, $gecos);
	my $sum = $gecos_entries[$field];
	if ( $users{$sum} ) { delete $users{$sum}; }

	foreach my $adminsum (@ignoresums)
		{
		# Prevent the admins' accounts from being created a second time
		if ( $users{$adminsum} ) { delete $users{$adminsum}; }
		}
	}

foreach my $newuser ( keys %users )
	{
	print OUTPUT "$users{$newuser}{'name'}\t$users{$newuser}{'realname'}\t$users{$newuser}{'class'}\t$users{$newuser}{'checksum'}\n";
	}
